@echo off
REM Created by Tyge Lovset Sept. 2016 (tycho)
REM flac.exe and lossywav.exe are released under the GPL license (see gpl.txt)
REM

setlocal
set ver=1.3
REM Use tools (flac, metaflac, lossywav) in %PATH%:
set tools=
REM Uncomment next line to use tools in same folder as this script only:
REM set "tools=%~dp0"

if "%~1"=="" (
  echo lossyflac v%ver%. Copyright 2016 tycho.
  echo.
  echo Usage: %~n0 inputfile... [lossywav-options] [@ flac-options]
  echo        %~n0 folder [lossywav-options] [@ flac-options]
  echo        %~n0 --help
  goto final
)
if "%~1"=="--help" (
  echo.
  echo lossyflac v%ver%. Copyright 2016 tycho.
  echo.
  echo Convert between WAV/FLAC and lossy.flac. Resulting flac files contains 
  echo tags and picture art from input flac files. Checking tools:
  echo.
  "%tools%lossyWAV.exe" --version
  "%tools%metaflac.exe" --version
  "%tools%flac.exe" --version
  echo.
  echo Usage: %~nx0 inputfile... [lossywav-options] [@ flac-options]
  echo        %~nx0 folder [lossywav-options] [@ flac-options]
  echo.
  echo Examples
  echo    %~n0 *.flac -q P @ -8                       create lossy.flacs in same dir
  echo    %~n0 *.wav -o lwdir                         create lossy.flac files in lwdir
  echo    %~n0 *.flac -o lwdir -C -f                  create lossy+corr. flac files 
  echo    %~n0 lwdir\*.lossy.flac -M -o flacdir       merge lossy+corr files back to lossless
  echo    %~n0 - -C -o song.lossy.flac                create lossy+corr. from std input
  echo    %~n0 song.lossy.flac -M                     merge back to lossless song.flac 
  echo    %~n0 Music -o Lossy -C                      convert all flacs in Music recursively
  echo    %~n0 Lossy -o Music2 -M                     reconstruct flacs from Lossy recursively
  goto final
)

set files=
set count=0
set recurse=0
:files_loop
  if "%~1"=="" goto files_done
  if "%~1"=="@" goto files_done
  set "f=%~1"
  if not "%~1"=="-" if "%f:~0,1%"=="-" goto files_done
  set files=%files% "%~1"
  set /a count=count+1
  shift& goto files_loop
:files_done

if %count%==0 (
  echo ERROR: a filename or "-" must be specified as first argument.
  goto final
)

set args=
set merge=0
:args_loop
  if "%~1"=="-M" if "%merge%"=="0" set merge=1
  if "%~1"=="--merge" if "%merge%"=="0" set merge=1
  if "%~1"=="@" if "%merge%"=="0" set merge=@
  if not "%~1"=="" (set args=%args% "%~1"& shift& goto args_loop)
REM done

set "workdir=%TEMP%\lflac_%RANDOM%%RANDOM%"
mkdir "%workdir%"
if %count%==1 if exist %files%\ set recurse=1
if %recurse%==0 for %%F in (%files%) do call :single_file "%%~F" %args%
if %recurse%==0 goto endnormal
  if not "%merge%"=="1" dir /s /b /a-d %files%\*.flac|findstr /v ".lossy.flac"|findstr /v ".lwcdf.flac"> "%workdir%\@lossyflac_in"
  if "%merge%"=="1" dir /s /b /a-d %files%\*.lossy.flac > "%workdir%\@lossyflac_in"
  for /f "usebackq delims=" %%F in ("%workdir%\@lossyflac_in") do call :single_file "%%~F" %args%
:endnormal
rmdir /s /q "%workdir%"
:final
endlocal
goto :eof


:single_file
setlocal
set lw_opts=
set fl_opts=
set "input=%~1"
set "iname=%~n1"
for %%F in ("%iname%") do set "ibname=%%~nF"
set "idir=%~dp1"
set "outdir=%~dp1"
if %recurse%==1 for %%F in (%files%) do set "outdir=%%~F"
set "oname=%~n1"
set output=
set force=
set operation=NORMAL
set flacInput=0
if /I "%~x1"==".flac" set flacInput=1
if /I "%~x1"==".fla" set flacInput=1

shift
:lw_loop
  shift
  if "%~0"=="" goto start
  if "%~0"=="@" goto fl_loop
  if "%~0"=="-f" (set force=-f& goto lw_loop)
  if "%~0"=="--force" (set force=-f& goto lw_loop)
  if "%~0"=="-C" set operation=CORRECTION
  if "%~0"=="--correction" set operation=CORRECTION
  if "%~0"=="-M" set operation=MERGE
  if "%~0"=="--merge" set operation=MERGE
  if "%~0"=="-o" (set "outdir=%~1"& shift& goto lw_loop)
  if "%~0"=="--outdir" (set "outdir=%~1"& shift& goto lw_loop)
  set "lw_opts=%lw_opts% %0"
goto lw_loop
:fl_loop
  shift
  if "%~0"=="" goto start
  if "%~0"=="-f" (set force=-f& goto fl_loop)
  if "%~0"=="--force" (set force=-f& goto fl_loop)
  if "%~0"=="-o" (set "outdir=%~dp1"& set "oname=%~n1"& set "output=%~1"& shift& goto fl_loop)  
  if "%~0"=="--output-name" (set "outdir=%~dp1"& set "oname=%~n1"& set "output=%~1"& shift& goto fl_loop)
  set "fl_opts=%fl_opts% %0"
goto fl_loop

:start
REM strip final backslash
if "%outdir:~-1%"=="\" set "outdir=%outdir:~0,-1%"

if %recurse%==0 goto recursedone
  for /d %%D in (%files%\) do set "xfile=%%~fD"
  for %%F in ("%input%") do set "xinput=%%~dpF"
  call set subdir=%%xinput:%xfile%=%%
  if not "%output%"=="" for %%F in ("%output%") do set "output=%%~dpF%subdir%%%~nxF"
  set "outdir=%outdir%\%subdir%"
  if "%outdir:~-1%"=="\" set "outdir=%outdir:~0,-1%"
:recursedone

if not exist "%outdir%\" mkdir "%outdir%"  
if not exist "%outdir%\" (
  echo ERROR: Output folder "%outdir%\" could not be created. Skipping "%input%".
  goto finish
)

for %%F in ("%oname%") do if /I "%%~xF"==".lossy" set "oname=%%~nF"
if "%input%"=="-" set "iname=%oname%"
if "%output%"=="" (
  if %operation%==MERGE (set "output=%outdir%\%oname%.flac") else (set "output=%outdir%\%oname%.lossy.flac")
)

REM call :debug
call :%operation%
:finish
endlocal
echo.
goto :eof


:NORMAL
if %flacInput%==1 (
  "%tools%flac.exe" -d "%input%" -o - --silent | "%tools%lossyWAV.exe" - %lw_opts% --stdout --stdinname "%oname%.wav" | "%tools%flac.exe" - -o "%output%" -b 512 --silent %fl_opts% %force%
  call :tags "%input%" "%output%"
) else (
  "%tools%lossyWAV.exe" "%input%" %lw_opts% --stdout --stdinname "%oname%.wav" | "%tools%flac.exe" - -o "%output%" -b 512 --silent %fl_opts% %force%
)
goto :eof


:CORRECTION
if "%force%"=="" if exist "%output%" (
  echo ERROR: output file "%output%" already exists. Use -f to override.
  goto :eof
)
if %flacInput%==1 (
  "%tools%flac.exe" "%input%" -d -o - --silent | "%tools%lossyWAV.exe" - %lw_opts% --stdinname "%oname%.wav" --outdir "%workdir%" --force
) else (
  "%tools%lossyWAV.exe" "%input%" %lw_opts% --stdinname "%oname%.wav" --outdir "%workdir%" --force
)
"%tools%flac.exe" "%workdir%\%oname%.lossy.wav" --totally-silent --delete-input-file --keep-foreign-metadata -b 512 %fl_opts% --force -o "%output%"
"%tools%flac.exe" "%workdir%\%oname%.lwcdf.wav" --totally-silent --delete-input-file --keep-foreign-metadata %fl_opts% --force -o "%outdir%\%oname%.lwcdf.flac"
if %flacInput%==1 call :tags "%input%" "%output%" "%outdir%\%oname%.lwcdf.flac"
goto :eof


:MERGE
if %flacInput%==1 (
  if not exist "%idir%%ibname%.lwcdf.flac" goto :eof
  if "%force%"=="" if exist "%output%" (
    echo ERROR: output file "%output%" already exists. Use -f to override.
    goto :eof
  )
  "%tools%flac.exe" "%input%" -d --totally-silent --keep-foreign-metadata -o "%workdir%\%oname%.lossy.wav" --force
  "%tools%flac.exe" "%idir%%ibname%.lwcdf.flac" -d --totally-silent --keep-foreign-metadata -o "%workdir%\%oname%.lwcdf.wav" --force

  "%tools%lossyWAV.exe" "%workdir%\%oname%.lossy.wav" %lw_opts% --outdir "%workdir%" --force
  del "%workdir%\%oname%.lossy.wav" "%workdir%\%oname%.lwcdf.wav"
  "%tools%flac.exe" "%workdir%\%oname%.wav" --delete-input-file %fl_opts% --force -o "%output%"
  call :tags "%input%" "%output%"
) else (
  echo ERROR: lossyflac merges flac'ed lossyWAV files only. Use lossyWAV directly for wav files.
)
goto :eof


:tags
set tag="%workdir%\%oname%_tag.txt"
set pic="%workdir%\%oname%_pic.jpg"
"%tools%metaflac.exe" --export-tags-to=%tag% --export-picture-to=%pic% %1 2>nul
if exist %pic% (
  "%tools%metaflac.exe" --import-tags-from=%tag% --import-picture-from=%pic% %2
  REM Tag the correction file too. Overkill?
  if not "%~3"=="" "%tools%metaflac.exe" --import-tags-from=%tag% %3
  del %tag% %pic%
) else (
  "%tools%metaflac.exe" --import-tags-from=%tag% %2 %3
  del %tag%
)
goto :eof


:debug
echo input: "%input%"
echo idir: "%idir%"
echo oname: "%oname%"
echo outdir: "%outdir%"
echo output: "%output%"
echo flacInput: %flacInput%
echo operation: %operation%
echo lw_opts: %lw_opts%
echo fl_opts: %fl_opts%
echo.
goto :eof
